<?php
/**
 * @package MediaWiki
 */
# Copyright (C) 2004 Brion Vibber <brion@pobox.com>
# http://www.mediawiki.org/
#
# Copyright (C) 2007 Kungliga Tekniska Högskolan
# (Royal Institute of Technology, Stockholm, Sweden).
# http://www.kth.se/
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# http://www.gnu.org/copyleft/gpl.html

/**
 * Authentication plugin for trusting an environment variable
 * containing a username.
 *
 * External usernames are mapped to local usernames in a configurable
 * way. Note that the mapping can NOT be case sensitive. Local
 * accounts may also be available. The user chooses which account to
 * use, if multiple options are available.
 *
 * @package MediaWiki
 */

require_once( 'AuthPlugin.php' );

function TestAuthPluginSetup() {
	global $wgHooks;
	global $wgAuth;

	$wgAuth = new TestAuthPlugin();

	$wgHooks['PersonalUrls'][] = 'TestAuthPluginPersonalUrls';
}

function TestAuthPluginPersonalUrls(&$personal_urls, $title) {
	global $wgEnvAuthLoginUrl;

	$personal_urls['login']['href'] = $wgEnvAuthLoginUrl;
	$personal_urls['anonlogin']['href'] = $wgEnvAuthLoginUrl;

	// XXX We'd like to use this link to log out both from the
	//     wiki and from whatever service provided the environment
	//     variable we used to verify the login.
	//$personal_urls['logout'] = null;
}

class TestAuthPlugin extends AuthPlugin {
	/**
	 * Check whether there exists a user account with the given name.
	 * The name will be normalized to MediaWiki's requirements, so
	 * you might need to munge it (for instance, for lowercase initial
	 * letters).
	 *
	 * @param $username String: username.
	 * @return bool
	 * @public
	 */
	function userExists( $username ) {
		global $wgEnvAuthDomain;

		// We assume that if they can authenticate, they're
		// allowed to log in.
		// echo "<br>\nXXXXXXXXXXX userExists<br>\nDomain: " . $this->domain . " == " . $wgEnvAuthDomain . "<br>\n";
		if ($this->domain == $wgEnvAuthDomain) {
			return true;
		}
		return false;
	}

	/**
	 * Check if a username+password pair is a valid login.
	 * The name will be normalized to MediaWiki's requirements, so
	 * you might need to munge it (for instance, for lowercase initial
	 * letters).
	 *
	 * @param $username String: username.
	 * @param $password String: user password.
	 * @return bool
	 * @public
	 */
	function authenticate( $username, $password ) {
		global $wgEnvAuthDomain, $wgEnvAuthVariable, $wgEnvAuthVarPrefix, $wgEnvAuthVarSuffix;

		// echo "<br>\nXXXXXXXXXXX<br>\n";
		// echo "Domain: " . $wgEnvAuthDomain . "<br>\n";
		
		if ($this->domain == $wgEnvAuthDomain) {
			if (isset($_SERVER[$wgEnvAuthVariable]) && $_SERVER[$wgEnvAuthVariable] != "") {
				$extuser = $_SERVER[$wgEnvAuthVariable];
				// echo "Extuser: " . $extuser . "<br>\n";
				$localuser = $wgEnvAuthVarPrefix . $extuser . $wgEnvAuthVarSuffix;
				// echo "localuser: " . $localuser . "<br>\n";
				return strtolower($username) == strtolower($localuser);
			}
		}
		return false;
	}

	/**
	 * Modify options in the login template.
	 *
	 * @param $template UserLoginTemplate object.
	 * @public
	 */
	function modifyUITemplate( &$template ) {
		global $wgEnvAuthUseLocal, $wgEnvAuthDomain;

		# Override this!
		$template->set( 'usedomain', true );
		if ($wgEnvAuthUseLocal) {
			$template->set( 'domainnames', array( $wgEnvAuthDomain, "local" ) );
		} else {
			$template->set( 'domainnames', array( $wgEnvAuthDomain ) );
		}
	}

	/**
	 * Set the domain this plugin is supposed to use when authenticating.
	 *
	 * @param $domain String: authentication domain.
	 * @public
	 */
// inherited
// 	function setDomain( $domain ) {
// 		$this->domain = $domain;
// 	}

	/**
	 * Check to see if the specific domain is a valid domain.
	 *
	 * @param $domain String: authentication domain.
	 * @return bool
	 * @public
	 */
	function validDomain( $domain ) {
		// echo "<br>\nXXXXXXXXXXX validDomain<br>\n";
		global $wgEnvAuthDomain, $wgEnvAuthUseLocal;

		if ($domain == $wgEnvAuthDomain)
			return true;
		if ($domain == "local" && $wgEnvAuthUseLocal)
			return true;
		return false;
	}

	/**
	 * When a user logs in, optionally fill in preferences and such.
	 * For instance, you might pull the email address or real name from the
	 * external user database.
	 *
	 * The User object is passed by reference so it can be modified; don't
	 * forget the & on your function declaration.
	 *
	 * @param User $user
	 * @public
	 */
	function updateUser( &$user ) {
		// Actually, this should be part of another class,
		// layered on top of this one.
		// This class doesn't create accounts.
		//global $wgEnvAuthEmailSuffix;

		//if (isset($wgEnvAuthEmailSuffix)) {
		//	$user->setEmail($_SERVER[$wgEnvAuthVariable] . $wgEnvAuthEmailSuffix);
		//	$user->mEmailAuthenticated = wfTimestampNow();
		//	$user->setToken();
		//}

		// Need to save new user_token in the database.
		// Not sure it's the right place but it seems to work...
		$user->saveSettings();

		return true;
	}


	/**
	 * Return true if the wiki should create a new local account automatically
	 * when asked to login a user who doesn't exist locally but does in the
	 * external auth database.
	 *
	 * If you don't automatically create accounts, you must still create
	 * accounts in some way. It's not possible to authenticate without
	 * a local account.
	 *
	 * This is just a question, and shouldn't perform any actions.
	 *
	 * @return bool
	 * @public
	 */
	function autoCreate() {
		return false;
	}

	/**
	 * Can users change their passwords?
	 *
	 * @return bool
	 */
	function allowPasswordChange() {
		global $wgEnvAuthUseLocal;

		// Allow local users to change their passwords.
		return $wgEnvAuthUseLocal;
	}

	/**
	 * Set the given password in the authentication database.
	 * Return true if successful.
	 *
	 * @param $password String: password.
	 * @return bool
	 * @public
	 */
// inherit
// 	function setPassword( $password ) {
// 		// XXX Uh, not the local user database then... hmm?
// 		//     Will returning false here cause password change
// 		//     for local users to fail?
// 		return true;
// 	}

	/**
	 * Update user information in the external authentication database.
	 * Return true if successful.
	 *
	 * @param $user User object.
	 * @return bool
	 * @public
	 */
	function updateExternalDB( $user ) {
		// Really nothing we can do.
		return false;
	}

	/**
	 * Check to see if external accounts can be created.
	 * Return true if external accounts can be created.
	 * @return bool
	 * @public
	 */
// inherit
// 	function canCreateAccounts() {
// 		return false;
// 	}

	/**
	 * Add a user to the external authentication database.
	 * Return true if successful.
	 *
	 * @param User $user
	 * @param string $password
	 * @return bool
	 * @public
	 */
	function addUser( $user, $password ) {
		// Can't do this.
		return false;
	}


	/**
	 * Return true to prevent logins that don't authenticate here from being
	 * checked against the local database's password fields.
	 *
	 * This is just a question, and shouldn't perform any actions.
	 *
	 * @return bool
	 * @public
	 */
	// XXX this doesn't work, it seems...
	function strict() {
		global $wgEnvAuthUseLocal;

		return !$wgEnvAuthUseLocal;
	}

	/**
	 * When creating a user account, optionally fill in preferences and such.
	 * For instance, you might pull the email address or real name from the
	 * external user database.
	 *
	 * The User object is passed by reference so it can be modified; don't
	 * forget the & on your function declaration.
	 *
	 * @param $user User object.
	 * @public
	 */
	// See comment in updateUser().
	//function initUser( &$user ) {
	//	// This'll work, right?
	//	$this->updateUser( $user );
	//}

	// XXX I don't think we need this, we munge the case elsewhere...
	/**
	 * If you want to munge the case of an account name before the final
	 * check, now is your chance.
	 */
	function getCanonicalName( $username ) {
		global $wgContLang;
		return $wgContLang->ucfirst($username);
	}
}

?>